@@ -15,20 +15,25 @@
 # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 from abc import abstractmethod
 import numpy as np
+
+from quocslib.Controls import Controls
 from quocslib.communication.AllInOneCommunication import AllInOneCommunication
 from quocslib import __VERSION__ as QUOCSLIB_VERSION
 from datetime import datetime
 
+from quocslib.gradientfreemethods.DirectSearchMethod import DirectSearchMethod
+
 INITIAL_FOM: float = 10**10
 
 
 class OptimizationAlgorithm:
-    init_status: bool = False
-    FoM_maximum: float = 1e10
+    init_status: bool
     xx: np.array
     alg_iteration_number: int
     iteration_number: int
     FoM_dict: dict
+    dsm_obj: DirectSearchMethod
+    controls: Controls
 
     def __init__(self, communication_obj: AllInOneCommunication = None, optimization_dict: dict = None):
         """
@@ -58,7 +63,7 @@ class OptimizationAlgorithm:
         # Goal FoM
         self.FoM_goal = alg_parameters.setdefault("FoM_goal", None)
         # initialize the optimizaiton start time
-        self.optimizaiton_start_time = 0
+        self.optimization_start_time = None
         # Maximization or minimization
         # optimization_direction
         self.optimization_direction = alg_parameters.setdefault("optimization_direction", "minimization")
@@ -71,7 +76,7 @@ class OptimizationAlgorithm:
                       "only, but {0} is provided".format(self.optimization_direction)
             self.comm_obj.print_logger(message=message, level=40)
             raise TypeError
-        self.best_FoM = self.optimization_factor * (-1.0) * INITIAL_FOM
+        self.best_FoM = self.FoM_maximum = self.optimization_factor * (-1.0) * INITIAL_FOM
         message = "The optimization direction is {0}".format(self.optimization_direction)
         self.comm_obj.print_logger(message=message, level=20)
 
@@ -88,7 +93,7 @@ class OptimizationAlgorithm:
         # Notify it to the client
         self.comm_obj.update_init_msg_server(upd_name=self.comm_obj.client_job_name)
         # set start time of optimization
-        self.optimizaiton_start_time = datetime.now()
+        self.optimization_start_time = datetime.now()
 
     def _routine_call(self, optimized_control_parameters: np.array, iterations: int) -> float:
         """
@@ -121,14 +126,14 @@ class OptimizationAlgorithm:
         # check total optimization time limit
         if self.total_time_lim < 10**10:
             curr_time = datetime.now()
-            time_passed = (curr_time - self.optimizaiton_start_time).total_seconds() / 60.0
+            time_passed = (curr_time - self.optimization_start_time).total_seconds() / 60.0
 
             if time_passed >= self.total_time_lim:
                 self.higher_order_terminate_reason = "Maximum optimization runtime reached"
                 self.dsm_obj.sc_obj.is_converged = True
                 self.stop_optimization()
 
-        # self.optimizaiton_start_time = datetime.now()
+        # self.optimization_start_time = datetime.now()
         # Check if the optimization is still running
         is_running = self.comm_obj.get_user_running()
         if not is_running:
@@ -162,16 +167,40 @@ class OptimizationAlgorithm:
         # add FoM to FoM track for stopping criteria
         try:
             self.dsm_obj.sc_obj.add_to_FoM_track(self.FoM_dict["FoM"])
-        except:
-            message = "FoM track for stopping criteria could not be updated!"
-            self.comm_obj.print_logger(message, level=30)
+        except Exception as ex:
+            ### example for exception logging
+            # template = "An exception of type {0} occurred. Arguments:\n{1!r}"
+            # message = template.format(type(ex).__name__, ex.args)
+
+            ### Ignore this if we are using GRAPE
+            if type(self).__name__ == "GRAPEAlgorithm":
+                pass
+            # elif type(ex).__name__ == "AttributeError":
+            #     message = "FoM track for stopping criteria could not be updated because " \
+            #               "there is no stopping criteria object!"
+            #     self.comm_obj.print_logger(message, level=30)
+            else:
+                message = "FoM track for stopping criteria could not be updated!"
+                self.comm_obj.print_logger(message, level=30)
 
         # check the advanced stopping criteria
         try:
             self.dsm_obj.sc_obj.check_advanced_stopping_criteria()
-        except:
-            message = "Advanced stopping criteria could not be checked!"
-            self.comm_obj.print_logger(message, level=30)
+        except Exception as ex:
+            ### example for exception logging
+            # template = "An exception of type {0} occurred. Arguments:\n{1!r}"
+            # message = template.format(type(ex).__name__, ex.args)
+
+            ### Ignore this if we are using GRAPE
+            if type(self).__name__ == "GRAPEAlgorithm":
+                pass
+            # elif type(ex).__name__ == "AttributeError":
+            #     message = "Advanced stopping criteria could not be checked because " \
+            #               "there is no stopping criteria object!"
+            #     self.comm_obj.print_logger(message, level=30)
+            else:
+                message = "Advanced stopping criteria could not be checked!"
+                self.comm_obj.print_logger(message, level=30)
 
         # Return the figure of merit, i.e. a real number, to the optimal based algorithm
         return -1.0 * self.optimization_factor * self.FoM_dict["FoM"]
@@ -208,6 +237,7 @@ class OptimizationAlgorithm:
         pulses, time grids, and parameters"""
         raise NotImplementedError("Must override method in the Optimal Algorithm class")
 
+    # TODO Make the below function no more public !
     @abstractmethod
     def _get_final_results(self) -> dict:
         """The optimal algorithm gives back a dictionary with useful results"""
@@ -221,6 +251,11 @@ class OptimizationAlgorithm:
         """Function to stop the optimization (inner direct search algorithm)"""
         self.comm_obj.set_is_running_state(value=False)
 
+    def get_best_controls(self) -> dict:
+        """Return the best pulses_list, time_grids_list, and parameters_list found so far"""
+        pulses_list, time_grids_list, parameters_list = self.controls.get_controls_lists(self.controls.get_mean_value())
+        return {"pulses": pulses_list, "parameters": parameters_list, "timegrids": time_grids_list}
+
     def end(self) -> None:
         """Finalize the transmission with  the client"""
         # Check client update
