@@ -21,7 +21,6 @@ from quocslib.utils.AbstractDump import AbstractDump
 
 
 class BestDump(AbstractDump):
-
     def __init__(self, results_path: str = ".", date_time: str = ".", **kwargs):
         """
         Dumping class for controls and other data which should be the most useful option for most users.
@@ -70,7 +69,7 @@ class BestDump(AbstractDump):
         np.savez(controls_path, **full_dict)
 
         if "iteration_number" in full_dict:
-            iteration_path = os.path.join(self.results_path, self.date_time + "_funt_eval_of_best_controls.txt")
+            iteration_path = os.path.join(self.results_path, self.date_time + "_funct_eval_of_best_controls.txt")
             with open(iteration_path, 'w') as f:
                 f.write(str(full_dict["iteration_number"]))
 
